using System;
using System.IO;
using System.Web.UI;
using ICSharpCode.SharpZipLib.Zip.Compression;
using ICSharpCode.SharpZipLib.Zip.Compression.Streams;

namespace ViewStateCompressionDemo
{
    /// <summary>
    /// Base page from which all of our code-behinds inherit.
    /// </summary>
    public class BasePage : Page
    {
        private const int BUFFER_SIZE = 65536;
        private int viewStateCompression = Deflater.NO_COMPRESSION;

        public int ViewStateCompression
        {
            get { return viewStateCompression; }
            set { viewStateCompression = value; }
        }

        protected override void SavePageStateToPersistenceMedium(Object state)
        {
            if (ViewStateCompression == Deflater.NO_COMPRESSION)
            {
                base.SavePageStateToPersistenceMedium(state);
                return;
            }

            Object viewState = state;
            if (state is Pair)
            {
                Pair statePair = (Pair) state;
                PageStatePersister.ControlState = statePair.First;
                viewState = statePair.Second;
            }

            using (StringWriter writer = new StringWriter())
            {
                new LosFormatter().Serialize(writer, viewState);
                string base64 = writer.ToString();
                byte[] compressed = Compress(Convert.FromBase64String((base64)));
                PageStatePersister.ViewState = Convert.ToBase64String(compressed);
            }
            PageStatePersister.Save();
        }

        protected override Object LoadPageStateFromPersistenceMedium()
        {
            if (viewStateCompression == Deflater.NO_COMPRESSION)
                return base.LoadPageStateFromPersistenceMedium();

            PageStatePersister.Load();
            String base64 = PageStatePersister.ViewState.ToString();
            byte[] state = Decompress(Convert.FromBase64String(base64));
            string serializedState = Convert.ToBase64String(state);

            object viewState = new LosFormatter().Deserialize(serializedState);
            return new Pair(PageStatePersister.ControlState, viewState);
        }

        private byte[] Compress(byte[] bytes)
        {
            using (MemoryStream memoryStream = new MemoryStream(BUFFER_SIZE))
            {
                Deflater deflater = new Deflater(ViewStateCompression);
                using (Stream stream = new DeflaterOutputStream(memoryStream, deflater, BUFFER_SIZE))
                {
                    stream.Write(bytes, 0, bytes.Length);
                }
                return memoryStream.ToArray();
            }
        }

        private byte[] Decompress(byte[] bytes)
        {
            using (MemoryStream byteStream = new MemoryStream(bytes))
            {
                using (Stream stream = new InflaterInputStream(byteStream))
                {
                    using (MemoryStream memory = new MemoryStream(BUFFER_SIZE))
                    {
                        byte[] buffer = new byte[BUFFER_SIZE];
                        while (true)
                        {
                            int size = stream.Read(buffer, 0, BUFFER_SIZE);
                            if (size <= 0)
                                break;

                            memory.Write(buffer, 0, size);
                        }
                        return memory.ToArray();
                    }
                }
            }
        }
    }
}